<?php

namespace App\DataTables;

use Carbon\Carbon;
use App\Models\TransactionReport;
use Yajra\DataTables\Html\Button;
use Illuminate\Support\Facades\DB;
use Yajra\DataTables\Services\DataTable;

class GallonByStateDataTable extends DataTable
{
    public function dataTable($query)
    {
        return datatables()
            ->eloquent($query)
            ->editColumn('TotalRebate', function ($row) {
                return number_format($row->{'TotalRebate'}, 2);
            })
            ->editColumn('AvgPrice', function ($row) {
                return number_format($row->{'AvgPrice'}, 2);
            })
            ->addColumn('Gallons', function ($row) {
                return $row->gallons;
            })
            ->editColumn('Gallons', function ($row) {
                return number_format($row->gallons, 2);
            })
            ->addColumn('RANGE', function ($row) {
                $start_date = '';
                $end_date = '';
                if (!empty($this->request()->get('start_date'))) {
                    $start_date = $this->request()->get('start_date');
                } else {
                    $start_date = Carbon::now()->startOfMonth();
                }
                if (!empty($this->request()->get('end_date'))) {
                    $end_date = $this->request()->get('end_date');
                } else {
                    $end_date = Carbon::now()->endOfMonth();
                }

                return date('m/d/Y', strtotime($start_date)) .' - '. date('m/d/Y', strtotime($end_date));
            })
            ->addColumn('action', 'gallonbystate.action')
            ->rawColumns(['Gallons', 'action']);
    }

    public function query(TransactionReport $model)
    {
        $start_date = '';
        $end_date = '';
        if (!empty($this->request()->get('start_date'))) {
            $start_date = $this->request()->get('start_date');
        } else {
            $start_date = Carbon::now()->startOfMonth();
        }
        if (!empty($this->request()->get('end_date'))) {
            $end_date = $this->request()->get('end_date');
        } else {
            $end_date = Carbon::now()->endOfMonth();
        }

        $query = $model->newQuery()
            ->select(
                DB::raw('`Location State/Province`, SUM(`Quantity`) as gallons, `Transaction Date`'),
                DB::raw('(SUM(Amount)/SUM(Quantity)) AS AvgPrice'),
                DB::raw('(AVG(Quantity)) AS TotalRebate')
                )
            ->where('Sub Fleet', getAuthMemberId())
            ->whereBetween('Transaction Date', [$start_date, $end_date])
            ->groupBy('Location State/Province')->orderBy('Transaction Date', 'DESC');

        return $query;
    }

    public function html()
    {
        return $this->builder()
            ->setTableId('gallonbystate-table')
            ->columns($this->getColumns())
            ->minifiedAjax()
            ->dom('Bfrtip')
            ->orderBy(1, 'desc')
            ->pageLength(25)
            ->setTableAttributes([
                'class' => 'table table-striped table-row-bordered table-row-dashed gy-4 align-middle fw-bold dataTable',
            ])
            ->headerCallback('
            function(thead, data, start, end, display){
                $(thead).addClass("thead-bg-color")
                $(thead).find("th").addClass("fs-7 text-gray-500 text-uppercase");
            }
        ')
            ->footerCallback('
            function(row, data, start, end, display) {
                var api = this.api();
                var intVal = function(i){
                    return typeof i === "string" ? i.replace(/[\$,]/g, "") * 1 : typeof i === "number" ? i : 0;
                };

                var total = api.column(1).data().reduce(function(a, b){
                    return intVal(a) + intVal(b);
                }, 0);

                $("#displayGallonTotal").html(total.toLocaleString(undefined, {minimumFractionDigits: 2}));
            }
        ')
            ->createdRow('
            function(row, data, dataIndex) {
                $(row).addClass("fs-6");
                if (data.ChainID == "2") {
                    $(row).find("td").css("color", "red");
                }
            }
        ')
            ->buttons(
                Button::make('excel'),
                Button::make('csv'),
                Button::make('print'),
                Button::make('reset'),
                Button::make('reload')
            )
            ->drawCallback('
            function() {
                $(".dataTables_filter").hide();
            }
        ')
            ->initComplete('
            function(settings, json) {
                $(".dt-buttons").addClass("d-none");
            }
        ');
    }

    protected function getColumns()
    {
        return [
            'Location State/Province' => ['title' => 'State'],
            'Gallons' => [
                'title' => 'Gallons',
                'orderable' => false,
                'searchable' => true,
                'exportable' => true,
                'printable' => true,
                'width' => '30%',
            ],
            'RANGE' => [
                'title' => 'RANGE',
                'orderable' => true,
                'searchable' => true,
                'exportable' => true,
                'printable' => true,
                'width' => '30%',
            ],
            'Avg Price' => ['title' => 'Avg Price'],
            'Avg Rebate' => ['title' => 'Avg Rebate'],
        ];
    }

    protected function filename()
    {
        return 'GallonByState_' . date('YmdHis');
    }
}
