<?php

namespace App\DataTables;

use App\Models\TransactionReport;
use Yajra\DataTables\Html\Button;
use Yajra\DataTables\Services\DataTable;
use Illuminate\Support\Carbon;
use DB;

class TAPetroGallonsDataTable extends DataTable
{
    public function dataTable($query)
    {
        return datatables()
            ->eloquent($query)
            ->editColumn('TotalRebate', function ($row) {
                return number_format($row->{'TotalRebate'}, 2);
            })
            ->editColumn('AvgPrice', function ($row) {
                return number_format($row->{'AvgPrice'}, 2);
            })
            ->editColumn('TotalCharges', function ($row) {
                return number_format($row->{'TotalCharges'}, 2);
            })
            ->editColumn('TotalGallons', function ($row) {
                return number_format($row->{'TotalGallons'}, 2);
            });
    }

    public function query(TransactionReport $model)
    {
        $start = "";
        $end = "";
        if (!empty($this->request()->get('start_date'))) {
            $start = $this->request()->get('start_date');
        } else {
            $start = Carbon::now()->startOfMonth();
        }

        if (!empty($this->request()->get('end_date'))) {
            $end = $this->request()->get('end_date');
        } else {
            $end = Carbon::now()->endOfMonth();
        }

        $query = $model->newQuery()->select(
            DB::raw('COUNT(*) as Total'),
            'Location State/Province',
            DB::raw('SUM(Quantity) as TotalGallons'),
            DB::raw('SUM(Amount) as TotalCharges'),
            DB::raw('(SUM(Amount)/SUM(Quantity)) AS AvgPrice'),
            DB::raw('(SUM(Quantity)*0.20) AS TotalRebate')
        )
            ->whereBetween('Transaction Date', [$start, $end])
            ->where('ChainID', '=', 3)
            ->whereIn('Product Code', ['ULSD', 'FUEL', 'ULSR'])
            ->where('Sub Fleet', getAuthMemberId())
            ->groupBy('Location State/Province')
            ->orderByDesc('TotalGallons');
        $search = $this->request()->get('search');
        $card_status = $this->request()->get('card_status');

        if (!empty($search)) {
            $query->having('Location State/Province', 'LIKE', "%$search%")
                ->orHaving(DB::raw('COUNT(*)'), 'LIKE', "%$search%")
                ->orHaving(DB::raw('SUM(Quantity)'), 'LIKE', "%$search%")
                ->orHaving(DB::raw('SUM(Amount)'), 'LIKE', "%$search%")
                ->orHaving(DB::raw('(SUM(Amount)/SUM(Quantity))'), 'LIKE', "%$search%")
                ->orHaving(DB::raw('(SUM(Quantity)*0.20)'), 'LIKE', "%$search%");
        }

        return $query;
    }

    public function html()
    {
        return $this->builder()
            ->setTableId('TAPetroGallons')
            ->columns($this->getColumns())
            ->minifiedAjax()
            ->dom('Bfrtip')
            ->orderBy(1, 'desc')
            ->pageLength(25)
            ->setTableAttributes([
                'class' => 'table table-row-bordered table-row-dashed gy-4 align-middle fw-bold dataTable no-footer',
            ])
            ->headerCallback('
            function(thead, data, start, end, display){
                $(thead).find("th").addClass("fs-7 text-gray-500 text-uppercase");
            }
        ')
            ->createdRow('
            function(row, data, dataIndex) {
                $(row).addClass("fs-6");
            }
        ')
            ->buttons(
                Button::make('excel'),
                Button::make('csv'),
                Button::make('print'),
                Button::make('reset'),
                Button::make('reload')
            )->drawCallback('
            function() {
                $(".dataTables_filter").hide();
            }
            ')->initComplete('
            function(settings, json) {
                $(".dt-buttons").addClass("d-none");
            }
        ');
    }

    protected function getColumns()
    {
        return [
            'Location State/Province' => ['title' => 'State'],
            'Total' => ['title' => 'Total'],
            'TotalGallons' => ['title' => 'TotalGallons'],
            'TotalCharges' => ['title' => 'TotalCharges'],
            'AvgPrice' => ['title' => 'AvgPrice'],
            'TotalRebate' => ['title' => 'TotalRebate'],
        ];
    }

    protected function filename()
    {
        return 'TAPetroGallons_' . date('YmdHis');
    }
}
